package io.educative.soap;

import java.io.IOException;

import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.transform.TransformerException;

import org.springframework.ws.WebServiceMessage;
import org.springframework.ws.client.core.WebServiceMessageCallback;
import org.springframework.ws.soap.saaj.SaajSoapMessage;

/**
 * 
 * <code>
 * &lt;wsse:Security xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd" wsse:mustUnderstand="1"&gt; <br/>
  &nbsp;&nbsp;&lt;wsse:UsernameToken&gt; <br/>
  &nbsp;&nbsp;&nbsp;&nbsp;&lt;wsse:Username&gt;testuser&lt;/wsse:Username&gt; <br/>
  &nbsp;&nbsp;&nbsp;&nbsp;&lt;wsse:Password Type="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText"&gt;testpass&lt;/wsse:Password&gt; <br/>
  &nbsp;&nbsp;&lt;/wsse:UsernameToken&gt; <br/>
&lt;/wsse:Security&gt;
 * </code>
 *
 */
public class AuthHeaderRequestCallback implements WebServiceMessageCallback {

	private final String username;
	private final String password;

	public AuthHeaderRequestCallback(String username, String password) {
		this.username = username;
		this.password = password;
	}

	@Override
	public void doWithMessage(WebServiceMessage message) throws IOException, TransformerException {

		try {

			SaajSoapMessage saajSoapMessage = (SaajSoapMessage) message;

			SOAPMessage soapMessage = saajSoapMessage.getSaajMessage();

			SOAPPart soapPart = soapMessage.getSOAPPart();

			SOAPEnvelope soapEnvelope = soapPart.getEnvelope();

			SOAPHeader soapHeader = soapEnvelope.getHeader();

			Name headerElementName = soapEnvelope.createName("Security", "wsse",
					"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd");
			SOAPHeaderElement authHeader = soapHeader.addHeaderElement(headerElementName);

			SOAPElement usernameToken = authHeader.addChildElement("UsernameToken", "wsse");

			SOAPElement usernamElement = usernameToken.addChildElement("Username", "wsse");

			usernamElement.addTextNode(username);

			SOAPElement passwordElement = usernameToken.addChildElement("Password", "wsse");

			passwordElement.addTextNode(password);

			soapMessage.saveChanges();
		} catch (SOAPException e) {
			throw new RuntimeException(e);
		}
	}
}
