package io.educative.soap;

import javax.annotation.PostConstruct;

import org.apache.http.HttpHost;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.util.ClassUtils;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.springframework.ws.transport.WebServiceMessageSender;
import org.springframework.ws.transport.http.HttpComponentsMessageSender;

import io.educative.soap_automation.GetStudentsRequest;

@Configuration
public class WebServiceClient {

	private static final String PROXY_HOST = System.getProperty("proxy.host", "proxyhost");
	private final Jaxb2Marshaller marshaller = new Jaxb2Marshaller();

	@PostConstruct
	public void init() throws Exception {
		marshaller.setPackagesToScan(ClassUtils.getPackageName(GetStudentsRequest.class));
		marshaller.afterPropertiesSet();
	}

	@Bean
	public WebServiceTemplate getWebServiceTemplate() {
		return new WebServiceTemplate(marshaller);

	}

	public WebServiceMessageSender getMessageSenderwithProxySettings() {
		RequestConfig config = RequestConfig
				.custom()
				.setProxy(new HttpHost(PROXY_HOST))
				.build();

		CloseableHttpClient client = HttpClients
				.custom()
				.setDefaultRequestConfig(config)
				.build();

		return new HttpComponentsMessageSender(client);
	}

}
