package io.educative.soap.tests;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertNotNull;
import static org.testng.Assert.assertTrue;

import org.springframework.ws.soap.client.SoapFaultClientException;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import io.educative.soap.helpers.StudentServiceHelper;
import io.educative.soap_automation.CreateStudentResponse;
import io.educative.soap_automation.DeleteStudentResponse;
import io.educative.soap_automation.GetStudentsResponse;
import io.educative.soap_automation.UpdateStudentResponse;

public class TestIntegrationFlow extends BaseTest {

	private StudentServiceHelper helper;

	@BeforeClass
	public void initHelper() {
		helper = new StudentServiceHelper(webServiceTemplate);
	}

	@Test
	public void testCreateUpdateDeleteStudent() {

		CreateStudentResponse createStudentResponse = helper.createStudent("Sam", "Bailey", "Female");

		// Validating response is not null
		assertNotNull(createStudentResponse, "CreateStudentResponse is null");

		// Printing the response XML
		printResponse(createStudentResponse);

		// Created Student ID
		long id = createStudentResponse.getStudent().getId();

		// Fetching the created Student response
		GetStudentsResponse getStudentsResponse = helper.getStudentById(id);

		// Validating response is not null and contains more than one object
		assertNotNull(getStudentsResponse, "GetStudentsResponse is null");
		assertTrue(!getStudentsResponse.getStudents().isEmpty(), "students list is empty");
		assertEquals(getStudentsResponse.getStudents().get(0).getFirstName(), "Sam", "first name");

		printResponse(getStudentsResponse);

		// Update the Student with information
		UpdateStudentResponse updateStudentResponse = helper.updateStudent(id, "Johnny", "Doe", "Male");

		// Validating response not null and the first name as given in the request
		assertNotNull(updateStudentResponse, "UpdateStudentResponse is null");
		assertEquals(updateStudentResponse.getStudent().getFirstName(), "Johnny");

		printResponse(updateStudentResponse);

		// Fetching the created Student response
		GetStudentsResponse getStudentsResponseAfterUpdate = helper.getStudentById(id);

		assertNotNull(getStudentsResponseAfterUpdate, "GetStudentsResponse is null");
		assertTrue(!getStudentsResponseAfterUpdate.getStudents().isEmpty(), "students list is empty");

		printResponse(getStudentsResponseAfterUpdate);

		// Deleting the Student
		DeleteStudentResponse deleteStudentResponse = helper.deleteStudent(id);

		// Validating response is not null and isDeleted() true on successful delete
		assertNotNull(deleteStudentResponse, "DeleteStudentResponse is null");
		assertTrue(deleteStudentResponse.isDeleted(), "Student not deleted");

		printResponse(deleteStudentResponse);

		// Fetching the deleted Student and expecting SoapFaultClientException
		try {
			helper.getStudentById(id);
		} catch (SoapFaultClientException e) {
			assertEquals(e.getMessage(), String.format("student with id '%s' not found", id));
		}
	}
}